// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:ui';

import 'package:flutter/widgets.dart';
import 'package:wyatt_ui_components/src/core/utils/multi_color.dart';
import 'package:wyatt_ui_components/src/domain/entities/buttons/button_style.dart';

class FlatButtonStyle extends ButtonStyle<FlatButtonStyle> {
  const FlatButtonStyle({
    this.label,
    super.radius,
    super.padding,
    super.foregroundColors,
    super.backgroundColors,
    super.borderColors,
    super.stroke,
    super.shadow,
  });

  /// Merges non-null `b` attributes in `a`
  static FlatButtonStyle? merge(
    FlatButtonStyle? a,
    FlatButtonStyle? b,
  ) {
    if (b == null) {
      return a?.copyWith();
    }
    if (a == null) {
      return b.copyWith();
    }

    return a.copyWith(
      label: b.label,
      radius: b.radius,
      padding: b.padding,
      foregroundColors: b.foregroundColors,
      backgroundColors: b.backgroundColors,
      borderColors: b.borderColors,
      stroke: b.stroke,
      shadow: b.shadow,
    );
  }

  /// Used for interpolation.
  static FlatButtonStyle? lerp(
    FlatButtonStyle? a,
    FlatButtonStyle? b,
    double t,
  ) {
    if (a == null || b == null) {
      return null;
    }
    // b.copyWith to return b attributes even if they are not lerped
    return b.copyWith(
      label: TextStyle.lerp(a.label, b.label, t),
      foregroundColors: MultiColor.lerp(
        a.foregroundColors,
        b.foregroundColors,
        t,
      ),
      backgroundColors: MultiColor.lerp(
        a.backgroundColors,
        b.backgroundColors,
        t,
      ),
      borderColors: MultiColor.lerp(
        a.borderColors,
        b.borderColors,
        t,
      ),
      radius: BorderRadiusGeometry.lerp(a.radius, b.radius, t),
      padding: EdgeInsetsGeometry.lerp(a.padding, b.padding, t),
      stroke: lerpDouble(a.stroke, b.stroke, t),
      shadow: BoxShadow.lerp(a.shadow, b.shadow, t),
    );
  }

  /// Label text style
  ///
  /// Default to `TextTheme.labelLarge`
  final TextStyle? label;

  @override
  FlatButtonStyle? mergeWith(FlatButtonStyle? other) =>
      FlatButtonStyle.merge(this, other);

  @override
  FlatButtonStyle? copyWith({
    TextStyle? label,
    BorderRadiusGeometry? radius,
    EdgeInsetsGeometry? padding,
    MultiColor? foregroundColors,
    MultiColor? backgroundColors,
    MultiColor? borderColors,
    double? stroke,
    BoxShadow? shadow,
  }) =>
      FlatButtonStyle(
        label: label ?? this.label,
        radius: radius ?? this.radius,
        padding: padding ?? this.padding,
        foregroundColors: foregroundColors ?? this.foregroundColors,
        backgroundColors: backgroundColors ?? this.backgroundColors,
        borderColors: borderColors ?? this.borderColors,
        stroke: stroke ?? this.stroke,
        shadow: shadow ?? this.shadow,
      );
}
